@@ -6,7 +6,7 @@ use warnings;
 
 BEGIN {
 	$Error::TypeTiny::AUTHORITY = 'cpan:TOBYINK';
-	$Error::TypeTiny::VERSION   = '1.012003';
+	$Error::TypeTiny::VERSION   = '1.016010';
 }
 
 $Error::TypeTiny::VERSION =~ tr/_//d;
@@ -70,7 +70,14 @@ sub new {
 }
 
 sub throw {
+	my $next = $_[0]->can( 'throw_cb' );
+	splice( @_, 1, 0, undef );
+	goto $next;
+}
+
+sub throw_cb {
 	my $class = shift;
+	my $callback = shift;
 	
 	my ( $level, @caller, %ctxt ) = 0;
 	while (
@@ -106,13 +113,13 @@ sub throw {
 		);
 	}
 	
-	die(
-		our $LastError = $class->new(
-			context     => \%ctxt,
-			stack_trace => $stack,
-			@_,
-		)
+	our $LastError = $class->new(
+		context     => \%ctxt,
+		stack_trace => $stack,
+		@_,
 	);
+	
+	$callback ? $callback->( $LastError ) : die( $LastError );
 } #/ sub throw
 
 sub message     { $_[0]{message} ||= $_[0]->_build_message }
@@ -198,6 +205,13 @@ Constructs an exception and passes it to C<die>.
 
 Automatically populates C<context> and C<stack_trace> if appropriate.
 
+=item C<< throw_cb($callback, %attributes) >>
+
+Constructs an exception and passes it to C<< $callback >> which should
+be a coderef; if undef, uses C<die>.
+
+Automatically populates C<context> and C<stack_trace> if appropriate.
+
 =back
 
 =head2 Attributes
@@ -298,7 +312,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2021 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2022 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
