@@ -10,7 +10,7 @@ BEGIN {
 
 BEGIN {
 	$Types::Common::Numeric::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Common::Numeric::VERSION   = '1.012003';
+	$Types::Common::Numeric::VERSION   = '1.016010';
 }
 
 $Types::Common::Numeric::VERSION =~ tr/_//d;
@@ -200,9 +200,9 @@ for my $base ( qw/Num Int/ ) {
 				);
 			}
 			push @whines, sprintf(
-				"length(%s) is %d",
+				"%s is %s",
 				$varname,
-				length( $value ),
+				$value,
 			);
 			return \@whines;
 		},
@@ -342,7 +342,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2021 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2022 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
