@@ -12,7 +12,7 @@ BEGIN {
 
 BEGIN {
 	$Types::Standard::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Standard::VERSION   = '1.012005';
+	$Types::Standard::VERSION   = '1.016010';
 }
 
 $Types::Standard::VERSION =~ tr/_//d;
@@ -130,7 +130,8 @@ my $maybe_load_modules = sub {
 	my $code = pop;
 	if ( $Type::Tiny::AvoidCallbacks ) {
 		$code = sprintf(
-			'do { package Type::Tiny; %s; %s }',
+			'do { %s %s; %s }',
+			$Type::Tiny::SafePackage,
 			join( '; ', map "use $_ ()", @_ ),
 			$code,
 		);
@@ -681,9 +682,74 @@ my $_Optional = $meta->add_type(
 	}
 );
 
+my $_slurpy;
+$_slurpy = $meta->add_type(
+	{
+		name                 => "Slurpy",
+		slurpy               => 1,
+		parent               => $_item,
+		constraint_generator => sub {
+			my $self  = $_slurpy;
+			my $param = @_ ? Types::TypeTiny::to_TypeTiny(shift) : $_any;
+			Types::TypeTiny::is_TypeTiny( $param )
+				or _croak(
+				"Parameter to Slurpy[`a] expected to be a type constraint; got $param" );
+			
+			return $self->create_child_type(
+				slurpy          => 1,
+				display_name    => $self->name_generator->( $self, $param ),
+				parameters      => [ $param ],
+				constraint      => sub { $param->check( $_[0] ) },
+				_build_coercion => sub {
+					my $coercion = shift;
+					$coercion->add_type_coercions( @{ $param->coercion->type_coercion_map } )
+						if $param->has_coercion;
+					$coercion->freeze;
+				},
+				$param->can_be_inlined
+					? ( inlined => sub { $param->inline_check( $_[1] ) } )
+					: (),
+			);
+		},
+		deep_explanation => sub {
+			my ( $type, $value, $varname ) = @_;
+			my $param = $type->parameters->[0];
+			return [
+				sprintf( '%s is slurpy', $varname ),
+				@{ $param->validate_explain( $value, $varname ) },
+			];
+		},
+		my_methods => {
+			'unslurpy' => sub {
+				my $self  = shift;
+				$self->{_my_unslurpy} ||= $self->find_parent(
+					sub { $_->parent->{uniq} == $_slurpy->{uniq} }
+				)->type_parameter;
+			},
+			'slurp_into' => sub {
+				my $self  = shift;
+				my $parameters = $self->find_parent(
+					sub { $_->parent->{uniq} == $_slurpy->{uniq} }
+				)->parameters;
+				if ( $parameters->[1] ) {
+					return $parameters->[1];
+				}
+				my $constraint = $parameters->[0];
+				return 'HASH'
+					if $constraint->is_a_type_of( HashRef() )
+					or $constraint->is_a_type_of( Map() )
+					or $constraint->is_a_type_of( Dict() );
+				return 'ARRAY';
+			},
+		},
+	}
+);
+
 sub slurpy {
 	my $t = shift;
-	wantarray ? ( +{ slurpy => $t }, @_ ) : +{ slurpy => $t };
+	my $s = $_slurpy->of( $t );
+	$s->{slurpy} ||= 1;
+	wantarray ? ( $s, @_ ) : $s;
 }
 
 $meta->$add_core_type(
@@ -692,10 +758,7 @@ $meta->$add_core_type(
 		parent         => $_arr,
 		name_generator => sub {
 			my ( $s, @a ) = @_;
-			sprintf(
-				'%s[%s]', $s, join q[,],
-				map { ref( $_ ) eq "HASH" ? sprintf( "slurpy %s", $_->{slurpy} ) : $_ } @a
-			);
+			sprintf( '%s[%s]', $s, join q[,], @a );
 		},
 		constraint_generator => LazyLoad( Tuple => 'constraint_generator' ),
 		inline_generator     => LazyLoad( Tuple => 'inline_generator' ),
@@ -725,12 +788,16 @@ $meta->add_type(
 		parent         => $_hash,
 		name_generator => sub {
 			my ( $s, @p ) = @_;
-			my $l = ref( $p[-1] ) eq q(HASH) ? pop( @p )->{slurpy} : undef;
+			my $l = @p
+				&& Types::TypeTiny::is_TypeTiny( $p[-1] )
+				&& $p[-1]->is_strictly_a_type_of( Types::Standard::Slurpy() )
+				? pop(@p)
+				: undef;
 			my %a = @p;
 			sprintf(
 				'%s[%s%s]', $s,
 				join( q[,], map sprintf( "%s=>%s", $_, $a{$_} ), sort keys %a ),
-				$l ? ",slurpy $l" : ''
+				$l ? ",$l" : ''
 			);
 		},
 		constraint_generator => LazyLoad( Dict => 'constraint_generator' ),
@@ -1295,7 +1362,7 @@ A blessed object.
 
 =head2 Structured
 
-OK, so I stole some ideas from L<MooseX::Types::Structured>.
+Okay, so I stole some ideas from L<MooseX::Types::Structured>.
 
 =over
 
@@ -1343,13 +1410,13 @@ L<Type::Params> for a similar purpose to how it's used in B<Tuple>.)
 
 =back
 
-This module also exports a C<slurpy> function, which can be used as
-follows.
+This module also exports a B<Slurpy> parameterized type, which can be
+used as follows.
 
 It can cause additional trailing values in a B<Tuple> to be slurped
 into a structure and validated. For example, slurping into an arrayref:
 
-   my $type = Tuple[Str, slurpy ArrayRef[Int]];
+   my $type = Tuple[ Str, Slurpy[ ArrayRef[Int] ] ];
    
    $type->( ["Hello"] );                # ok
    $type->( ["Hello", 1, 2, 3] );       # ok
@@ -1357,7 +1424,7 @@ into a structure and validated. For example, slurping into an arrayref:
 
 Or into a hashref:
 
-   my $type2 = Tuple[Str, slurpy Map[Int, RegexpRef]];
+   my $type2 = Tuple[ Str, Slurpy[ Map[Int, RegexpRef] ] ];
    
    $type2->( ["Hello"] );                               # ok
    $type2->( ["Hello", 1, qr/one/i, 2, qr/two/] );      # ok
@@ -1365,21 +1432,27 @@ Or into a hashref:
 It can cause additional values in a B<Dict> to be slurped into a
 hashref and validated:
 
-   my $type3 = Dict[ values => ArrayRef, slurpy HashRef[Str] ];
+   my $type3 = Dict[ values => ArrayRef, Slurpy[ HashRef[Str] ] ];
    
    $type3->( { values => [] } );                        # ok
    $type3->( { values => [], name => "Foo" } );         # ok
    $type3->( { values => [], name => [] } );            # not ok
 
-In either B<Tuple> or B<Dict>, B<< slurpy Any >> can be used to indicate
+In either B<Tuple> or B<Dict>, B<< Slurpy[Any] >> can be used to indicate
 that additional values are acceptable, but should not be constrained in
 any way. 
 
-B<< slurpy Any >> is an optimized code path. Although the following are
+B<< Slurpy[Any] >> is an optimized code path. Although the following are
 essentially equivalent checks, the former should run a lot faster:
 
-   Tuple[Int, slurpy Any]
-   Tuple[Int, slurpy ArrayRef]
+   Tuple[ Int, Slurpy[Any] ]
+   Tuple[ Int, Slurpy[ArrayRef] ]
+
+A function C<< slurpy($type) >> is also exported which was historically
+how slurpy types were created.
+
+Outside of B<Dict> and B<Tuple>, B<< Slurpy[Foo] >> should just act the
+same as B<Foo>. But don't do that.
 
 =begin trustme
 
@@ -1389,7 +1462,7 @@ essentially equivalent checks, the former should run a lot faster:
 
 =head2 Objects
 
-OK, so I stole some ideas from L<MooX::Types::MooseLike::Base>.
+Okay, so I stole some ideas from L<MooX::Types::MooseLike::Base>.
 
 =over
 
@@ -1582,7 +1655,7 @@ C<< [1,{},2] >> and C<< [1,{},2,[]] >>.
 
 I think you understand B<CycleTuple> already.
 
-Currently B<Optional> and C<slurpy> parameters are forbidden. There are
+Currently B<Optional> and B<Slurpy> parameters are forbidden. There are
 fairly limited use cases for them, and it's not exactly clear what they
 should mean.
 
@@ -1738,7 +1811,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2021 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2022 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
